from pathlib import Path

from aws_cdk import RemovalPolicy
from aws_cdk import aws_certificatemanager as acm
from aws_cdk import aws_cloudfront as cloudfront
from aws_cdk import aws_cloudfront_origins as origins
from aws_cdk import aws_route53 as route53
from aws_cdk import aws_route53_targets as targets
from aws_cdk import aws_s3 as s3
from aws_cdk import aws_s3_deployment as s3_deployment
from constructs import Construct


class StaticSite(Construct):
    """CDK construct for the infrastructure and deployment of static websites."""

    def __init__(
        self,
        *,
        scope: Construct,
        construct_id: str,
        site_domain_name: str,
        hosted_zone_id: str,
        hosted_zone_name: str,
        domain_certificate_arn: str,
        **kwargs,
    ):
        """
        Create the infrastructure for the static website.

        Args:
            scope: The scope in which to define this construct.
            construct_id: The scoped construct ID.
            site_domain_name: The domain name where this site will be hosted.
            hosted_zone_id: The ID of the hosted zone created for the domain.
                This should be created beforehand either manually or through CDK/CloudFormation.
            hosted_zone_name: The name of the hosted zone created for the domain.
            domain_certificate_arn: The ARN of the SSL certificate created in AWS Certificate Manager.
                Note, this needs to be created in us-east-1.
            **kwargs:
        """
        super().__init__(scope, construct_id, **kwargs)

        self._bucket = self._create_site_bucket(site_domain_name)
        hosted_zone = self._get_hosted_zone(
            hosted_zone_name=hosted_zone_name,
            hosted_zone_id=hosted_zone_id,
        )
        certificate = self._get_certificate(certificate_arn=domain_certificate_arn)
        self._distribution = self._create_cloudfront_distribution(
            bucket=self.bucket,
            site_domain_name=site_domain_name,
            certificate=certificate,
        )
        self._create_route53_record(
            hosted_zone=hosted_zone,
            site_domain_name=site_domain_name,
            distribution=self._distribution,
        )

    def deploy(self, asset_path: str):
        """Deploy the static assets to the bucket and invalidate the distribution."""
        asset = s3_deployment.Source.asset(asset_path)
        s3_deployment.BucketDeployment(
            self,
            "S3Deployment",
            sources=[asset],
            destination_bucket=self.bucket,
            distribution=self.distribution,
            distribution_paths=["/*"],
        )

    @property
    def bucket(self) -> s3.IBucket:
        """The private S3 bucket the static assets will be uploaded to."""
        return self._bucket

    @property
    def distribution(self) -> cloudfront.IDistribution:
        """The CloudFront distribution exposing the contents of the S3 bucket."""
        return self._distribution

    def _create_site_bucket(self, site_domain_name: str) -> s3.Bucket:
        return s3.Bucket(
            self,
            "SiteBucket",
            bucket_name=site_domain_name,
            encryption=s3.BucketEncryption.S3_MANAGED,
            block_public_access=s3.BlockPublicAccess.BLOCK_ALL,
            removal_policy=RemovalPolicy.DESTROY,
            auto_delete_objects=True,
        )

    def _create_cloudfront_distribution(
        self,
        *,
        bucket: s3.IBucket,
        site_domain_name: str,
        certificate: acm.ICertificate,
    ) -> cloudfront.IDistribution:
        # function_name is supplied to apply the workaround for https://github.com/aws/aws-cdk/issues/15523
        function_name = f"RouterFunction{self.node.addr}"

        function_code = Path(__file__).parent / "router.js"
        router_function = cloudfront.Function(
            self,
            function_name,
            code=cloudfront.FunctionCode.from_file(file_path=function_code.as_posix()),
            function_name=function_name,
        )

        return cloudfront.Distribution(
            self,
            "CloudFrontDistribution",
            default_behavior=cloudfront.BehaviorOptions(
                origin=origins.S3Origin(bucket),
                viewer_protocol_policy=cloudfront.ViewerProtocolPolicy.REDIRECT_TO_HTTPS,
                function_associations=[
                    cloudfront.FunctionAssociation(
                        event_type=cloudfront.FunctionEventType.VIEWER_REQUEST,
                        function=router_function,
                    )
                ],
            ),
            domain_names=[site_domain_name],
            certificate=certificate,
            default_root_object="index.html",
        )

    def _get_hosted_zone(
        self, *, hosted_zone_name: str, hosted_zone_id: str
    ) -> route53.IHostedZone:
        return route53.HostedZone.from_hosted_zone_attributes(
            self,
            "HostedZone",
            zone_name=hosted_zone_name,
            hosted_zone_id=hosted_zone_id,
        )

    def _create_route53_record(
        self,
        *,
        hosted_zone: route53.IHostedZone,
        site_domain_name: str,
        distribution: cloudfront.IDistribution,
    ) -> route53.ARecord:
        return route53.ARecord(
            self,
            "SiteAliasRecord",
            record_name=site_domain_name,
            zone=hosted_zone,
            target=route53.RecordTarget.from_alias(
                targets.CloudFrontTarget(distribution)
            ),
        )

    def _get_certificate(self, certificate_arn: str) -> acm.ICertificate:
        return acm.Certificate.from_certificate_arn(
            self,
            "SiteCertificate",
            certificate_arn=certificate_arn,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['static_site']

package_data = \
{'': ['*']}

install_requires = \
['aws-cdk-lib>=2.28,<3.0']

setup_kwargs = {
    'name': 'cdk-static-site',
    'version': '0.1.2',
    'description': 'CDK construct for static websites for CDK v2',
    'long_description': '# CDK Static Website Construct\n\nThis package contains a high-level CDK construct for describing the infrastructure of\na static website. The construct also features a `deploy` method to deploy static assets\nfor the website and invalidate the CloudFront distribution.\n\nThe CDK construct creates the private S3 bucket, where the website is hosted. The contents\nof the bucket are exposed through a CloudFront distribution. Rather than allowing public\naccess to the bucket, OAI is used to allow access from CloudFront only.\n\nFurther reading: https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/private-content-restricting-access-to-s3.html\n\nThis construct also creates the necessary Route 53 records to route requests to\nthe CloudFront distribution. It also configures the distribution with a supplied\nSSL certificate.\n\n## Pre-requisites\n\nThis construct works with recent versions of CDK v2. There are similar constructs\non PyPI for v1 - one of the motivations for this package was to support v2.\n\nIf you are not familiar with CDK, have a read [here](https://aws.amazon.com/cdk/).\n\nRegistering a domain name, creating the hosted zone in Route 53 and creating the SSL\ncertificate are not in scope for this construct. These can be created manually or\nthrough other automated ways on AWS.\n\nNote, the certificate needs to be created in North Virginia (us-east-1).\n\nOnce these are created, you should have all the necessary inputs: the domain name,\ndomain certificate ARN, the hosted zone ID and the hosted zone name.\n\n## How to use it?\n\nInstall the package using your favourite package manager, e.g.\n\n```shell\npip install cdk-static-site\n```\n\nWithin your stack, instantiate the construct\n```python\nfrom static_site import StaticSite\n\nsite = StaticSite(\n    scope=self,\n    construct_id="StaticSite",\n    site_domain_name=site_domain_name,\n    hosted_zone_id=hosted_zone_id,\n    hosted_zone_name=hosted_zone_name,\n    domain_certificate_arn=domain_certificate_arn,\n)\n```\n\nand finally, deploy the static contents by supplying the directory path\nto `deploy`:\n\n```python\nfrom pathlib import Path\n\ncontents_path = Path.cwd() / "public"  # or wherever your assets are located\nsite.deploy(contents_path.as_posix())\n```\n\n## Contribution\n\nIf you find any issues, or have suggestions to improve the construct, feel free to\nopen an issue/PR.\n',
    'author': 'David Steiner',
    'author_email': 'david_j_steiner@yahoo.co.nz',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/davidsteiner/cdk-static-site',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
